<?php

/**
 * @file
 * Definition of WoWHttpException.
 */

/**
 * Defines the WoWHttpException class.
 */
class WoWHttpException extends Exception {
  protected $request;
  protected $reason;
  protected $protocol;
  protected $headers;

  /**
   * Constructor.
   */
  public function __construct($response) {
    parent::__construct(isset($response->error) ? $response->error : $response->status_message, $response->code);
    $this->request = $response->request;
    if (isset($response->data)) {
      $this->reason = json_decode($response->data)->reason;
    }
    if (isset($response->protocol)) {
      $this->protocol = $response->protocol;
    }
    if (isset($response->headers)) {
      $this->headers = $response->headers;
    }
  }

  /**
   * Return request.
   *
   * @var string
   */
  public function getRequest() {
    return $this->request;
  }

  /**
   * Return data.
   *
   * @var string
   */
  public function getReason() {
    return $this->reason;
  }

  /**
   * Return protocol.
   *
   * @var string
   */
  public function getProtocol() {
    return $this->protocol;
  }

  /**
   * Return headers.
   *
   * @var array
   */
  public function getHeaders() {
    return $this->headers;
  }
}
