<?php

/**
 * @file
 * Definition of WoWHttpResponse.
 */

/**
 * Defines the WoWHttpResponse class.
 */
class WoWHttpResponse {

  /**
   * The date of the request used to sign authentication.
   *
   * @var Date
   */
  private $date;

  /**
   * The region with which the request was made.
   *
   * @var string
   */
  private $region;

  /**
   * The data returned by the service.
   *
   * @var string
   */
  private $data;

  /**
   * The data as array. This is primarily used by functions to create entities.
   *
   * @var array
   */
  private $dataArray;

  /**
   * The data as object. This is used to get the original message structure.
   *
   * @var object
   */
  private $dataObject;

  public function __construct($region, $data, $date) {
    $this->region = $region;
    $this->data = $data;
    $this->date = $date;
  }

  /**
   * Get the region.
   *
   * @return string
   *   The region with which the request was made.
   */
  public function getRegion() {
    return $this->region;
  }

  /**
   * Get the data.
   *
   * @param string $key
   *   (Optionnal) A key to return.
   *
   * @return array|mixed
   *   The data returned by the service as an array or a value if a key was given.
   */
  public function getArray($key = NULL) {
    if (empty($this->dataArray)) {
      $this->dataArray = drupal_json_decode($this->data);
    }
    return isset($key) && isset($this->dataArray[$key]) ? $this->dataArray[$key] : $this->dataArray;
  }

  /**
   * Get the data.
   *
   * @param string $key
   *   (Optionnal) A key to return.
   *
   * @return object|mixed
   *   The data returned by the service as an object or a value if a key was given.
   */
  public function getObject($key = NULL) {
    if (empty($this->dataObject)) {
      $this->dataObject = json_decode($this->data);
    }
    return isset($key) && isset($this->dataObject->{$key}) ? $this->dataObject->{$key} : $this->dataObject;
  }

  /**
   * Get the date.
   *
   * @return DateTime
   *   The date with which the request was made.
   */
  public function getDate() {
    return $this->date;
  }
}
