<?php

/**
 * @file
 * Entity related hooks for the wow_character module.
 */

/**
 * Delete a character.
 *
 * @deprecated Use $character->delete() instead.
 *
 * @param int $cid
 *   A character ID.
 */
function wow_character_delete($cid) {
  wow_character_delete_multiple(array($cid));
}

/**
 * Delete multiple characters.
 *
 * @deprecated Use entity_get_controller('wow_character')->delete($cids) instead.
 *
 * @param array $cids
 *   An array of character IDs.
 */
function wow_character_delete_multiple(array $cids) {
  if (!empty($cids)) {
    $characters = wow_character_load_multiple($cids);

    $transaction = db_transaction();
    try {
      foreach ($characters as $cid => $character) {
        module_invoke_all('wow_character_delete', $character);
        module_invoke_all('entity_delete', $character, 'wow_character');
        field_attach_delete('wow_character', $character);
      }

      // Delete after calling hooks so that they can query wow_characters table
      // as needed.
      db_delete('wow_characters')
      ->condition('cid', $cids, 'IN')
      ->execute();
    }
    catch (Exception $e) {
      $transaction->rollback();
      watchdog_exception('wow_character', $e);
      throw $e;
    }

    // Clear the static loading cache.
    entity_get_controller('wow_character')->resetCache();
  }
}

/**
 * Save changes to a character or add a new character.
 *
 * @deprecated Use $character->save() instead.
 *
 * @param object $character
 *   The $character object to be saved. If $character->cid is
 *   omitted (or $character->is_new is TRUE), a new character will be added.
 */
function wow_character_save($character) {
  $transaction = db_transaction();

  try {
    // Load the stored entity, if any.
    if (!empty($character->cid) && !isset($character->original)) {
      $character->original = entity_load_unchanged('wow_character', $character->cid);
    }

    field_attach_presave('wow_character', $character);

    // Determine if we will be inserting a new character.
    if (!isset($character->is_new)) {
      $character->is_new = empty($character->cid);
    }

    // Let modules modify the character before it is saved to the database.
    module_invoke_all('wow_character_presave', $character);
    module_invoke_all('entity_presave', $character, 'wow_character');

    // Save the character.
    if ($character->is_new) {
      // For new character, save new records for the character.
      drupal_write_record('wow_characters', $character);
      $op = 'insert';
    }
    else {
      // For existing characters, update the character record which matches
      // the value of $character->cid.
      drupal_write_record('wow_characters', $character, 'cid');
      $op = 'update';
    }

    // Save fields.
    $function = "field_attach_$op";
    $function('wow_character', $character);

    module_invoke_all('wow_character_' . $op, $character);
    module_invoke_all('entity_' . $op, $character, 'wow_character');

    // Clear internal properties.
    unset($character->is_new);
    unset($character->original);
    // Clear the static loading cache.
    entity_get_controller('wow_character')->resetCache(array($character->cid));

    // Ignore slave server temporarily to give time for the
    // saved character to be propagated to the slave.
    db_ignore_slave();
  }
  catch (Exception $exception) {
    $transaction->rollback();
    watchdog_exception('wow_character', $exception);
    throw $exception;
  }
}
