<?php

/**
 * @file
 * Contains funtions to interact with character module hooks.
 */

/**
 * Implements hook_character_presave().
 *
 * @ingroup character
 */
function wow_guild_rank_character_presave($character) {
  if (isset($character->status) && $character->status != $character->original->status) {

    // Check if this is the highest rank in the guild.
    $query = db_select('wow_characters', 'c')
      ->orderBy('rank')
      ->fields('c', array('cid'))
      ->condition('uid', $character->original->uid)
      ->condition('gid', variable_get('wow_guild')->gid)
      ->condition('cid', $character->original->cid, '<>')
      ->condition('status', TRUE);

    // If we are about to unblock this user, when no character have a higher
    // rank than this one, then he can become main.
    // If we are about to block this user, we need to select the next
    // candidate to become main.
    if ($character->status) {
      $query->condition('rank', $character->rank, '<');
    }

    $select = $query->execute();

    // Unblock the user.
    if ($character->status && !$select->rowCount()) {
      // No one is a better candidate than this character.
      $character->is_main = TRUE;

      // Clean-up other potential mains.
      $cids = db_select('wow_characters', 'c')
        ->fields('c', array('cid'))
        ->condition('is_main', TRUE)
        ->condition('uid', $character->original->uid)
        ->execute()
        ->fetchCol();

      foreach (wow_character_load_multiple($cids) as $alt) {
        $alt->original = clone $alt;
        $alt->is_main = FALSE;
        wow_character_save($alt);
      }
    }
    // Block the user.
    elseif (!$character->status) {
      // This character is being blocked and can't be main anymore.
      $character->is_main = FALSE;

      if ($select->rowCount()) {
        // This candidate is set as main.
        $candidate = wow_character_load($select->fetchColumn());
        $candidate->original = clone $candidate;
        $candidate->is_main = TRUE;
        wow_character_save($candidate);
      }
    }

  }
}
